#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

float   HeightMapTerra(vec3 point)
{
    noiseH          = 0.5;
    noiseLacunarity = 2.218281828459;
    noiseOffset     = 0.8;
    float massif, global;
    vec3  distort;
    vec3  p = point * mainFreq + Randomize;

    // Global landscape
    noiseOctaves = 5;
    distort = 0.35 * fBmVec3(p * 2.37);
    noiseOctaves = 4;
    distort += 0.005 * (1.0 - abs(fBmVec3(p * 132.3)));
    global = 1.0 - Cell3Noise(p + distort);

    // Venus-like structure
    float venus = 0.0;
    if (venusMagn > 0.05)
    {
        noiseOctaves = 4;
        distort = fBmVec3(point * 0.3) * 1.5;
        noiseOctaves = 6;
        venus = fBm((point + distort) * venusFreq) * venusMagn;
    }

    global += venus;
    global = (global - seaLevel) * 0.5 + seaLevel;

    // Dunes
    float dunes = 0.0;
    if (dunesSqrtDensity > 0.05)
    {
        noiseOctaves = 3;
        massif = Cell2Noise(point * 3.5 + 0.2 * fBmVec3(point*5.12));
        massif = 1.0 - smoothstep(0.0, 1.0, massif / dunesSqrtDensity);
        if (massif > 0.0)
        {
            noiseOctaves = 5.0;
            dunes = dunesFreq + fBm(p * 3.21);
            dunes = max(fBm(p * dunes) + 0.7, 0.0);
            dunes = dunesMagn * massif * dunes;
        }
    }

    // "Eroded" hills
    float hills = 0.0;
    if (hillsSqrtDensity > 0.05)
    {
        noiseOctaves = 3;
        massif = Cell2Noise(point * 4.0 + 0.2 * fBmVec3(point*5.12));
        massif = 1.0 - smoothstep(0.0, 1.0, massif / hillsSqrtDensity);
        if (massif > 0.0)
        {
            noiseOctaves = 10.0;
            noiseH       = 1.0;
            noiseOffset  = 1.0;
            hills = hillsMagn * massif * (1.5 - RidgedMultifractal(point * hillsFreq + Randomize, 2.0));
            noiseOctaves = 10.0;
            distort = 0.2 * fBmVec3(p * 15.2);
            vec2 cell = Cell3Noise2(25.51 * (p + distort));
            hills = smoothstep(0.0, 1.0, abs(cell.y - cell.x));
        }
    }

    // Canyons
    float canyon = 0.0;
    if (canyonSqrtDensity > 0.05)
    {
        noiseOctaves = 3;
        massif = Cell2Noise(point * 3.8 + 0.2 * fBmVec3(point*5.12));
        massif = 1.0 - smoothstep(0.0, 1.0, massif / canyonSqrtDensity);
        if (massif > 0.0)
        {
            noiseOctaves = 3;
            canyon = canyonFreq + 30.0 * fBm(p * 1.321);
            float canyonWidth = clamp(1.0 - 1.5 * massif, 0.3, 2.0);
            canyon = pow(abs(fBm(p * canyon)), canyonWidth);
            canyon = smoothstep(0.0, 1.0, (canyon - 0.5) * 6) - 1.0;
            canyon = canyon * massif * canyonMagn;
        }
    }

    global += canyon + (dunes + hills) * clamp(70 * (global - seaLevel), 0.0, 1.0);

    // Mare
    float mare = global;
    if (mareSqrtDensity > 0.05)
    {
        craterDistortion = 1.0;
        noiseOctaves = 6;  // Mare roundness distortion
        mare = MareNoise(point, global, 0.0);
    }

    // Craters
    float crater = 0.0;
    if (craterSqrtDensity > 0.05)
    {
        noiseOctaves = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor = -0.1;
        heightPeak  = 0.6;
        heightRim   = 1.0;
        crater = CraterNoise(point, 0.5 * craterMagn, craterFreq, craterSqrtDensity, craterOctaves);
        noiseOctaves = 10.0;
        noiseH       = 1.0;
        noiseOffset  = 1.0;
        crater = RidgedMultifractalDetail(point * 0.3 * montesFreq + Randomize, 2.0, 0.25 * crater);
    }

    // Mountains
    float montes = 0.0;
    if (montesSqrtDensity > 0.05)
    {
        noiseOctaves = 3;
        massif = Cell2Noise(point * 4.5 + 0.2 * fBmVec3(point*5.12));
        massif = 1.0 - smoothstep(0.0, 1.0, massif / montesSqrtDensity);
        massif = saturate((global - seaLevel - 0.05) / 0.07) * 2.0 - 1.0;
        massif = 1.0 - montes*montes;
        if (massif > 0.0)
        {
            noiseOctaves = 10.0;
            noiseH       = 1.0;
            noiseOffset  = 1.0;
            montes = montesMagn * massif * RidgedMultifractal(point * montesFreq + Randomize, 2.0);
        }
    }

    //float height = mare + crater + montes;
    float height = mare + crater + montes * clamp(5 * (global - seaLevel), 0.0, 1.0);

    // Assign a climate type
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    noiseH          = 0.5;
    noiseLacunarity = 2.218281828459;
    noiseOffset     = 0.8;
    float climate, latitude;
    if (tidalLock == 0.0)
    {
        latitude = abs(point.y);
        latitude += 0.15 * (fBm(point * 0.7 + Randomize) - 1.0);
        latitude = saturate(latitude);
    }
    else
    {
        latitude = 1.0 - point.x;
        latitude += 0.15 * (fBm(point * 0.7 + Randomize) - 1.0);
    }

    // Ice caps
    float edge;
    if (cloudsStyle == 1.0)
    {
        edge = saturate(15.0 * (latitude - latIceCaps));
        height = height * 0.1 + icecapHeight * edge;
    }
    else
    {
        edge = saturate(3.0 * (latitude - latIceCaps));
        height += icecapHeight * sqrt(edge);
    }

    return height;
}

void main()
{
    vec3  point = GetSurfacePoint();
    //float height = max((HeightMapTerra(point) - seaLevel) / (1.0 + montesMagn), 0.0);
    float height = HeightMapTerra(point);
    gl_FragColor = vec4(height);
}

#endif
